@@ -14,6 +14,39 @@
 #include "gam_error.h"
 #include "config.h"
 
+#ifdef HAVE_PTHREAD_H
+#include <pthread.h>
+
+static int is_threaded = -1;
+#endif
+
+/*
+ * Use weak symbols on gcc/linux to avoid forcing link with pthreads for
+ * non threaded apps using the gamin library.
+ * patches to extends this to other platforms/compilers welcome, note that
+ * this also affects configure.in
+ * #pragma weak .... might be more portable but may not work from shared
+ * libraries.
+ */
+#ifdef __GNUC__
+#ifdef linux
+#if (__GNUC__ == 3 && __GNUC_MINOR__ >= 3) || (__GNUC__ > 3)
+extern int pthread_mutexattr_init(pthread_mutexattr_t *attr)
+           __attribute((weak));
+extern int pthread_mutexattr_settype (pthread_mutexattr_t *__attr, int __kind)
+           __attribute((weak));
+extern int pthread_mutexattr_init (pthread_mutexattr_t *__attr)
+           __attribute((weak));
+extern int pthread_mutex_lock (pthread_mutex_t *__mutex)
+           __attribute((weak));
+extern int pthread_mutex_unlock (pthread_mutex_t *__mutex)
+           __attribute((weak));
+extern int pthread_mutexattr_destroy (pthread_mutexattr_t *__attr)
+           __attribute((weak));
+#endif
+#endif /* linux */
+#endif /* __GNUC__ */
+
 #define FAM_EVENT_SIZE (sizeof(FAMEvent))
 
 #ifdef GAMIN_DEBUG_API
@@ -50,8 +83,28 @@ struct GAMData {
     int req_nr;                 /* the number of running requests */
     int req_max;                /* the size of req_tab */
     GAMReqDataPtr *req_tab;     /* pointer to the array of requests */
+
+#ifdef HAVE_PTHREAD_H
+    pthread_mutex_t lock;	/* mutex protecting this structure,
+				   it's connection, and everything related */
+#endif
 };
 
+void gamin_data_lock(GAMDataPtr data)
+{
+#ifdef HAVE_PTHREAD_H
+    if (is_threaded > 0)
+	pthread_mutex_lock(&data->lock);
+#endif
+}
+
+void gamin_data_unlock(GAMDataPtr data)
+{
+#ifdef HAVE_PTHREAD_H
+    if (is_threaded > 0)
+	pthread_mutex_unlock(&data->lock);
+#endif
+}
 /************************************************************************
  *									*
  *		Request Data handling					*
@@ -207,6 +260,8 @@ gamin_data_add_req2(GAMDataPtr conn, const char *filename, int type,
     conn->req_tab[idx] = req;
     conn->req_nr++;
 
+    GAM_DEBUG(DEBUG_INFO, "Allocated request %d\n", reqno);
+
     return (req);
 }
 
@@ -318,9 +373,36 @@ gamin_data_del_req(GAMDataPtr conn, int reqno)
         memmove(&conn->req_tab[idx], &conn->req_tab[idx + 1],
                 (conn->req_nr - idx) * sizeof(GAMReqDataPtr));
     }
+
+    GAM_DEBUG(DEBUG_INFO, "Removed request %d\n", reqno);
+
     return (0);
 }
 
+/**
+ * gamin_data_cancel:
+ * @conn:  a connection data structure
+ *
+ * Cancel a connection
+ *
+ * Returns 0 or 1 in case or -1 in case of error.
+ */
+int
+gamin_data_cancel(GAMDataPtr conn, int reqno)
+{
+    int idx;
+    GAMReqDataPtr data;
+
+    idx = gamin_data_get_req_idx(conn, reqno);
+    if (idx < 0)
+        return (-1);
+    data = conn->req_tab[idx];
+    if (data->state == REQ_CANCELLED)
+        return(0);
+    data->state = REQ_CANCELLED;
+    return(1);
+}
+
 /**
  * gamin_data_get_req:
  * @conn:  a connection data structure
@@ -363,11 +445,38 @@ GAMDataPtr
 gamin_data_new(void)
 {
     GAMDataPtr ret;
+#ifdef HAVE_PTHREAD_H
+    pthread_mutexattr_t attr;
+#endif
 
     ret = (GAMDataPtr) malloc(sizeof(GAMData));
     if (ret == NULL)
         return (NULL);
     memset(ret, 0, sizeof(GAMData));
+    
+#ifdef HAVE_PTHREAD_H
+    if (is_threaded == -1) {
+        if ((pthread_mutexattr_init != NULL) &&
+	    (pthread_mutexattr_settype != NULL) &&
+	    (pthread_mutex_init != NULL) &&
+	    (pthread_mutexattr_destroy != NULL) &&
+	    (pthread_mutex_lock != NULL) &&
+	    (pthread_mutex_unlock != NULL) &&
+	    (pthread_mutexattr_destroy != NULL)) {
+	    is_threaded = 1;
+	    GAM_DEBUG(DEBUG_INFO, "Activating thread safety\n");
+	} else {
+	    GAM_DEBUG(DEBUG_INFO, "Not activating thread safety\n");
+	}
+    }
+    if (is_threaded > 0) {
+	pthread_mutexattr_init(&attr);
+	pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE_NP);
+	pthread_mutex_init(&ret->lock, &attr);
+	pthread_mutexattr_destroy(&attr);
+    }
+#endif
+
     ret->auth = 0;
     ret->reqno = 1;
     ret->evn_ready = 0;
@@ -398,6 +507,12 @@ gamin_data_free(GAMDataPtr conn)
         }
         free(conn->req_tab);
     }
+
+#ifdef HAVE_PTHREAD_H
+    pthread_mutex_unlock(&conn->lock);
+    pthread_mutex_destroy(&conn->lock);
+#endif
+
     free(conn);
 }
 
@@ -458,6 +573,12 @@ gamin_data_read_event(GAMDataPtr conn, FAMEvent * event)
     event->code = evn->type;
     conn->evn_ready = 0;
     conn->evn_read -= evn->len;
+    if (event->code == FAMAcknowledge) {
+        /*
+         * destroy the request internally
+         */
+        gamin_data_del_req(conn, evn->seq);
+    }
     if (conn->evn_read > 0) {
         /*
          * there was other events piggy-backed on the same read,
@@ -529,16 +650,22 @@ gamin_data_conn_event(GAMDataPtr conn, GAMPacketPtr evn)
         return (0);
     }
 
-    switch (req->type) {
+    switch (req->state) {
         case REQ_NONE:
         case REQ_SUSPENDED:
+            GAM_DEBUG(DEBUG_INFO,
+                      "Event: seq %d dropped, request type %d\n", evn->seq,
+                      req->type);
+            return (0);
         case REQ_CANCELLED:
+	    if (evn->type == FAMAcknowledge)
+	        break;
             GAM_DEBUG(DEBUG_INFO,
                       "Event: seq %d dropped, request type %d\n", evn->seq,
                       req->type);
             return (0);
         case REQ_INIT:
-            req->type = REQ_CONFIRMED;
+            req->state = REQ_CONFIRMED;
         case REQ_CONFIRMED:
             break;
     }
@@ -811,3 +938,4 @@ gamin_data_get_exists(GAMDataPtr conn)
         return(0);
     return(1);
 }
+
