@@ -10,7 +10,14 @@ G_BEGIN_DECLS
 typedef enum {
     GAMDnotifyCreate=1,
     GAMDnotifyDelete=2,
-    GAMDnotifyChange=3
+    GAMDnotifyChange=3,
+    GAMDnotifyFlowOn=4,
+    GAMDnotifyFlowOff=5,
+    GAMinotifyCreate=6,
+    GAMinotifyDelete=7,
+    GAMinotifyChange=8,
+    GAMinotifyFlowOn=9,
+    GAMinotifyFlowOff=10
 } GAMDebugEvent;
 
 void gam_debug_add(GamConnDataPtr conn, const char *value, int options);
