@@ -17,7 +17,7 @@
  */
 
 
-#include <config.h>
+#include "server_config.h"
 #define _GNU_SOURCE
 #include <fcntl.h>
 #include <signal.h>
@@ -42,6 +42,7 @@ typedef struct {
     char *path;
     int fd;
     int refcount;
+    int busy;
 } DNotifyData;
 
 static GHashTable *path_hash = NULL;
@@ -57,6 +58,30 @@ static GIOChannel *pipe_write_ioc = NULL;
 
 static gboolean have_consume_idler = FALSE;
 
+static void 
+gam_dnotify_data_debug (gpointer key, gpointer value, gpointer user_data)
+{
+    int deactivated;
+    DNotifyData *data = (DNotifyData *)value;
+
+    if (!data)
+        return;
+
+    deactivated = data->fd == -1 ? 1 : 0;
+
+    GAM_DEBUG(DEBUG_INFO, "dsub fd %d refs %d busy %d deactivated %d: %s\n", data->fd, data->refcount, data->busy, deactivated, data->path);
+}
+
+void
+gam_dnotify_debug ()
+{
+    if (path_hash == NULL)
+        return;
+
+    GAM_DEBUG(DEBUG_INFO, "Dumping dnotify subscriptions\n");
+    g_hash_table_foreach (path_hash, gam_dnotify_data_debug, NULL);
+}
+
 static DNotifyData *
 gam_dnotify_data_new(const char *path, int fd)
 {
@@ -65,6 +90,7 @@ gam_dnotify_data_new(const char *path, int fd)
     data = g_new0(DNotifyData, 1);
     data->path = g_strdup(path);
     data->fd = fd;
+    data->busy = 0;
     data->refcount = 1;
 
     return data;
@@ -78,20 +104,33 @@ gam_dnotify_data_free(DNotifyData * data)
 }
 
 static void
-gam_dnotify_directory_handler_internal(const char *path, gboolean added)
+gam_dnotify_directory_handler_internal(const char *path, pollHandlerMode mode)
 {
     DNotifyData *data;
     int fd;
 
-    if (added) {
-	GAM_DEBUG(DEBUG_INFO, "Adding %s to dnotify\n", path);
-    } else {
-        GAM_DEBUG(DEBUG_INFO, "Removing %s from dnotify\n", path);
+    switch (mode) {
+        case GAMIN_ACTIVATE:
+	    GAM_DEBUG(DEBUG_INFO, "Adding %s to dnotify\n", path);
+	    break;
+        case GAMIN_DESACTIVATE:
+	    GAM_DEBUG(DEBUG_INFO, "Removing %s from dnotify\n", path);
+	    break;
+	case GAMIN_FLOWCONTROLSTART:
+	    GAM_DEBUG(DEBUG_INFO, "Start flow control for %s\n", path);
+	    break;
+	case GAMIN_FLOWCONTROLSTOP:
+	    GAM_DEBUG(DEBUG_INFO, "Stop flow control for %s\n", path);
+	    break;
+	default:
+	    gam_error(DEBUG_INFO, "Unknown DNotify operation %d for %s\n",
+	              mode, path);
+	    return;
     }
 
     G_LOCK(dnotify);
 
-    if (added) {
+    if (mode == GAMIN_ACTIVATE) {
 
         if ((data = g_hash_table_lookup(path_hash, path)) != NULL) {
             data->refcount++;
@@ -123,25 +162,14 @@ gam_dnotify_directory_handler_internal(const char *path, gboolean added)
 #ifdef GAMIN_DEBUG_API
         gam_debug_report(GAMDnotifyCreate, path, 0);
 #endif
-    } else {
-	char *dir = path;
-
+    } else if (mode == GAMIN_DESACTIVATE) {
         data = g_hash_table_lookup(path_hash, path);
 
         if (!data) {
+	    GAM_DEBUG(DEBUG_INFO, "  not found !!!\n");
 
-	    dir = g_path_get_dirname(path);
-	    data = g_hash_table_lookup(path_hash, dir);
-
-            if (!data) {
-		GAM_DEBUG(DEBUG_INFO, "  not found !!!\n");
-
-		if (dir != NULL)
-		    g_free(dir);
-		G_UNLOCK(dnotify);
-		return;
-	    }
-	    GAM_DEBUG(DEBUG_INFO, "  not found using parent\n");
+	    G_UNLOCK(dnotify);
+	    return;
         }
 
         data->refcount--;
@@ -154,54 +182,90 @@ gam_dnotify_directory_handler_internal(const char *path, gboolean added)
             g_hash_table_remove(fd_hash, GINT_TO_POINTER(data->fd));
             gam_dnotify_data_free(data);
 #ifdef GAMIN_DEBUG_API
-	    gam_debug_report(GAMDnotifyDelete, dir, 0);
+	    gam_debug_report(GAMDnotifyDelete, path, 0);
 #endif
         } else {
 	    GAM_DEBUG(DEBUG_INFO, "  found decremented refcount: %d\n",
 	              data->refcount);
 #ifdef GAMIN_DEBUG_API
-            gam_debug_report(GAMDnotifyChange, dir, data->refcount);
+            gam_debug_report(GAMDnotifyChange, data->path, data->refcount);
 #endif
 	}
-	if ((dir != path) && (dir != NULL))
-	    g_free(dir);
+    } else if ((mode == GAMIN_FLOWCONTROLSTART) ||
+               (mode == GAMIN_FLOWCONTROLSTOP)) {
+	data = g_hash_table_lookup(path_hash, path);
+	if (!data) {
+	    GAM_DEBUG(DEBUG_INFO, "  not found !!!\n");
+
+	    G_UNLOCK(dnotify);
+	    return;
+        }
+        if (data != NULL) {
+	    if (mode == GAMIN_FLOWCONTROLSTART) {
+		if (data->fd >= 0) {
+		    close(data->fd);
+		    g_hash_table_remove(fd_hash, GINT_TO_POINTER(data->fd));
+		    data->fd = -1;
+		    GAM_DEBUG(DEBUG_INFO, "deactivated DNotify for %s\n",
+			      data->path);
+#ifdef GAMIN_DEBUG_API
+		    gam_debug_report(GAMDnotifyFlowOn, data->path, 0);
+#endif
+		}
+		data->busy++;
+	    } else {
+	        if (data->busy > 0) {
+		    data->busy--;
+		    if (data->busy == 0) {
+			fd = open(data->path, O_RDONLY);
+			if (fd < 0) {
+			    G_UNLOCK(dnotify);
+			    GAM_DEBUG(DEBUG_INFO,
+			              "Failed to reactivate DNotify for %s\n",
+				      data->path);
+			    return;
+			}
+			data->fd = fd;
+			g_hash_table_insert(fd_hash, GINT_TO_POINTER(data->fd),
+			                    data);
+			fcntl(fd, F_SETSIG, SIGRTMIN);
+			fcntl(fd, F_NOTIFY,
+			      DN_MODIFY | DN_CREATE | DN_DELETE | DN_RENAME |
+			      DN_ATTRIB | DN_MULTISHOT);
+			GAM_DEBUG(DEBUG_INFO, "Reactivated DNotify for %s\n",
+			          data->path);
+#ifdef GAMIN_DEBUG_API
+			gam_debug_report(GAMDnotifyFlowOff, path, 0);
+#endif
+		    }
+		}
+	    }
+	}
+    } else {
+	GAM_DEBUG(DEBUG_INFO, "Unimplemented operation\n");
     }
 
     G_UNLOCK(dnotify);
 }
 
 static void
-gam_dnotify_directory_handler(const char *path, gboolean added)
+gam_dnotify_directory_handler(const char *path, pollHandlerMode mode)
 {
     GAM_DEBUG(DEBUG_INFO, "gam_dnotify_directory_handler %s : %d\n",
-              path, added);
+              path, mode);
 
-    if ((!added) || (g_file_test(path, G_FILE_TEST_IS_DIR))) {
-	gam_dnotify_directory_handler_internal(path, added);
-    } else {
-	char *dir;
-
-	dir = g_path_get_dirname(path);
-	GAM_DEBUG(DEBUG_INFO, " not a dir using parent %s\n", dir);
-	gam_dnotify_directory_handler_internal(dir, added);
-	g_free(dir);
-    }
+    gam_dnotify_directory_handler_internal(path, mode);
 }
 
 static void
-gam_dnotify_file_handler(const char *path, gboolean added)
+gam_dnotify_file_handler(const char *path, pollHandlerMode mode)
 {
-    GAM_DEBUG(DEBUG_INFO, "gam_dnotify_file_handler %s : %d\n", path, added);
+    GAM_DEBUG(DEBUG_INFO, "gam_dnotify_file_handler %s : %d\n", path, mode);
     
     if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
-	gam_dnotify_directory_handler_internal(path, added);
+	gam_dnotify_directory_handler_internal(path, mode);
     } else {
-	char *dir;
-
-	dir = g_path_get_dirname(path);
-	GAM_DEBUG(DEBUG_INFO, " not a dir using parent %s\n", dir);
-	gam_dnotify_directory_handler_internal(dir, added);
-	g_free(dir);
+	GAM_DEBUG(DEBUG_INFO, " not a dir %s, failed !!!\n", path);
     }
 }
 
@@ -347,8 +411,9 @@ gam_dnotify_init(void)
 
     path_hash = g_hash_table_new(g_str_hash, g_str_equal);
     fd_hash = g_hash_table_new(g_direct_hash, g_direct_equal);
-    gam_poll_set_directory_handler(gam_dnotify_directory_handler);
-    gam_poll_set_file_handler(gam_dnotify_file_handler);
+    gam_poll_set_kernel_handler(gam_dnotify_directory_handler,
+                                gam_dnotify_file_handler,
+				GAMIN_K_DNOTIFY);
 
     GAM_DEBUG(DEBUG_INFO, "dnotify initialized\n");
 
