@@ -1,6 +1,5 @@
-
-#ifndef __MD_INOTIFY_H__
-#define __MD_INOTIFY_H__
+#ifndef __GAM_INOTIFY_H__
+#define __GAM_INOTIFY_H__
 
 #include <glib.h>
 #include "gam_poll.h"
@@ -12,7 +11,9 @@ gboolean   gam_inotify_init                  (void);
 gboolean   gam_inotify_add_subscription      (GamSubscription *sub);
 gboolean   gam_inotify_remove_subscription   (GamSubscription *sub);
 gboolean   gam_inotify_remove_all_for        (GamListener *listener);
+void       gam_inotify_debug                 (void); 
+gboolean   gam_inotify_is_running            (void);
 
 G_END_DECLS
 
-#endif /* __MD_INOTIFY_H__ */
+#endif /* __GAM_INOTIFY_H__ */
