@@ -3,8 +3,11 @@
 #define __GAM_NODE_H__
 
 #include <glib.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include "gam_event.h"
 #include "gam_subscription.h"
+#include "gam_fs.h"
 
 G_BEGIN_DECLS
 
@@ -13,16 +16,21 @@ typedef struct _GamNode GamNode;
 typedef gboolean (*GamSubFilterFunc) (GamSubscription *sub);
 
 struct _GamNode {
-	char *path;
+        /* the node informations proper */
+	char *path;		/* The file path */
+	GList *subs;		/* the list of subscriptions */
+	GNode *node;		/* pointer in the tree */
+	gboolean is_dir;	/* is that a directory or expected to be one */
+	int flags;		/* generic flags */
+	int poll_time;		/* How often this node should be polled */
+	gam_fs_mon_type mon_type; /* the type of notification that should be done */
 
-	GList *subs;
-	gpointer data;
-	GDestroyNotify data_destroy;
-
-	int flags;
-
-	GNode *node;
-	gboolean is_dir;
+        /* what used to be stored in a separate data structure */
+	int checks;
+	int pflags;		/* A combination of MON_xxx flags */
+	time_t lasttime;	/* Epoch of last time checking was done */
+	int flow_on_ticks;	/* Number of ticks while flow control is on */
+	struct stat sbuf;	/* The stat() informations in last check */
 };
 
 
