@@ -18,7 +18,7 @@
  */
 
 
-#include <config.h>
+#include "server_config.h"
 #include <unistd.h>
 #include <signal.h>
 #include <string.h>
@@ -39,11 +39,37 @@
 #ifdef ENABLE_DNOTIFY
 #include "gam_dnotify.h"
 #endif
+#ifdef ENABLE_KQUEUE
+#include "gam_kqueue.h"
+#endif
+#ifdef ENABLE_HURD_MACH_NOTIFY
+#include "gam_hurd_mach_notify.h"
+#endif
 #include "gam_excludes.h"
+#include "gam_fs.h"
+#include "gam_conf.h" 
 
 static int poll_only = 0;
 static const char *session;
 
+gboolean (*gam_backend_add_subscription) (GamSubscription *sub);
+gboolean (*gam_backend_remove_subscription) (GamSubscription *sub);
+gboolean (*gam_backend_remove_all_for)   (GamListener *listener);
+
+/**
+ * gam_exit:
+ *
+ * Call the shutdown routine, then just exit.
+ * This function is designed to be called from a
+ * signal handler.
+ */
+static void
+gam_exit(int signo) {
+	gam_shutdown();
+
+	exit(0);
+}
+
 /**
  * gam_shutdown:
  *
@@ -61,7 +87,16 @@ gam_shutdown(void) {
  */
 void
 gam_show_debug(void) {
+	gam_exclude_debug ();
+    gam_fs_debug ();
     gam_connections_debug();
+#ifdef ENABLE_INOTIFY
+    gam_inotify_debug ();
+#endif
+#ifdef ENABLE_DNOTIFY
+    gam_dnotify_debug ();
+#endif
+    gam_poll_debug();
 }
 
 /**
@@ -75,6 +110,7 @@ gam_show_debug(void) {
 gboolean
 gam_init_subscriptions(void)
 {
+	gam_conf_read ();
     gam_exclude_init();
 
     if (!poll_only) {
@@ -90,6 +126,18 @@ gam_init_subscriptions(void)
 	    return(TRUE);
 	}
 #endif
+#ifdef ENABLE_KQUEUE
+	if (gam_kqueue_init()) {
+	    GAM_DEBUG(DEBUG_INFO, "Using KQueue as backend\n");
+	    return(TRUE);
+	}
+#endif
+#ifdef ENABLE_HURD_MACH_NOTIFY
+	if (gam_hurd_notify_init()) {
+	    GAM_DEBUG(DEBUG_INFO, "Using Hurd Notify as backend\n");
+	    return(TRUE);
+	}
+#endif	
     }
     if (gam_poll_init()) {
 	GAM_DEBUG(DEBUG_INFO, "Using Poll as backend\n");
@@ -112,11 +160,36 @@ gam_init_subscriptions(void)
 gboolean
 gam_add_subscription(GamSubscription * sub)
 {
-    
-    if (sub == NULL)
-        return(FALSE);
+	const char *path = NULL;
 
-    return (gam_backend_add_subscription(sub));
+	if (sub == NULL)
+		return(FALSE);
+
+	path = gam_subscription_get_path (sub);
+
+	if (gam_exclude_check (path)) 
+	{
+		GAM_DEBUG(DEBUG_INFO, "g_a_s: %s excluded\n", path);
+		if (gam_inotify_is_running())
+			return gam_poll_add_subscription (sub);
+		else
+			return gam_backend_add_subscription(sub);
+	} else {
+		gam_fs_mon_type type;
+		type = gam_fs_get_mon_type (path);
+		if (type == GFS_MT_KERNEL) 
+		{
+			GAM_DEBUG(DEBUG_INFO, "g_a_s: %s using kernel monitoring\n", path);
+			return (gam_backend_add_subscription(sub));
+		}
+		else if (type == GFS_MT_POLL)
+		{
+			GAM_DEBUG(DEBUG_INFO, "g_a_s: %s using poll monitoring\n", path);
+			return gam_poll_add_subscription (sub);
+		}
+	}
+
+	return FALSE;
 }
 
 /**
@@ -129,7 +202,29 @@ gam_add_subscription(GamSubscription * sub)
 gboolean
 gam_remove_subscription(GamSubscription * sub)
 {
-    return (gam_backend_remove_subscription(sub));
+	const char *path = NULL;
+
+	if (sub == NULL)
+		return(FALSE);
+
+	path = gam_subscription_get_path (sub);
+
+	if (gam_exclude_check (path)) 
+	{
+		if (gam_inotify_is_running())
+			return gam_poll_remove_subscription (sub);
+		else
+			return gam_backend_remove_subscription(sub);
+	} else {
+		gam_fs_mon_type type;
+		type = gam_fs_get_mon_type (path);
+		if (type == GFS_MT_KERNEL)
+			return (gam_backend_remove_subscription(sub));
+		else if (type == GFS_MT_POLL)
+			return gam_poll_remove_subscription (sub);
+	}
+
+	return FALSE;
 }
 
 /**
@@ -180,6 +275,7 @@ gam_server_emit_one_event(const char *path, int node_is_dir,
 
     if (!gam_subscription_wants_event(sub, path, node_is_dir, event, force))
 	return;
+
     listener = gam_subscription_get_listener(sub);
     if (listener == NULL)
 	return;
@@ -204,10 +300,15 @@ gam_server_emit_one_event(const char *path, int node_is_dir,
 
     reqno = gam_subscription_get_reqno(sub);
 
-    if (gam_send_event(conn, reqno, event, subpath, len) < 0) {
-	GAM_DEBUG(DEBUG_INFO, "Failed to send event to PID %d\n",
-		  gam_connection_get_pid(conn));
-    }
+	if (gam_inotify_is_running())
+	{
+		gam_queue_event(conn, reqno, event, subpath, len);
+	} else {
+		if (gam_send_event(conn, reqno, event, subpath, len) < 0) {
+		GAM_DEBUG(DEBUG_INFO, "Failed to send event to PID %d\n",
+			  gam_connection_get_pid(conn));
+		}
+	}
 }
 
 /**
@@ -226,8 +327,7 @@ gam_server_emit_event(const char *path, int is_dir_node, GaminEventType event,
                       GList * subs, int force)
 {
     GList *l;
-    int pathlen, len;
-    const char *subpath;
+    int pathlen;
 
     if ((path == NULL) || (subs == NULL))
         return;
@@ -235,40 +335,7 @@ gam_server_emit_event(const char *path, int is_dir_node, GaminEventType event,
 
     for (l = subs; l; l = l->next) {
         GamSubscription *sub = l->data;
-        GamListener *listener;
-        GamConnDataPtr conn;
-        int reqno;
-
-        if (!gam_subscription_wants_event(sub, path, is_dir_node, event, force))
-            continue;
-        listener = gam_subscription_get_listener(sub);
-        if (listener == NULL)
-            continue;
-        conn = (GamConnDataPtr) gam_listener_get_service(listener);
-        if (conn == NULL)
-            continue;
-
-        /*
-         * When sending directory related entries, for items in the
-         * directory the FAM protocol removes the common direcory part.
-         */
-        subpath = path;
-        len = pathlen;
-        if (gam_subscription_is_dir(sub)) {
-            int dlen = gam_subscription_pathlen(sub);
-
-            if ((pathlen > dlen + 1) && (path[dlen] == '/')) {
-                subpath += dlen + 1;
-                len -= dlen + 1;
-            }
-        }
-
-        reqno = gam_subscription_get_reqno(sub);
-
-        if (gam_send_event(conn, reqno, event, subpath, len) < 0) {
-            GAM_DEBUG(DEBUG_INFO, "Failed to send event to PID %d\n",
-                      gam_connection_get_pid(conn));
-        }
+	gam_server_emit_one_event (path, is_dir_node, event, sub, force);
     }
 }
 
@@ -332,6 +399,10 @@ main(int argc, const char *argv[])
     }
 
     gam_error_init();
+    signal(SIGHUP, gam_exit);
+    signal(SIGINT, gam_exit);
+    signal(SIGQUIT, gam_exit);
+    signal(SIGTERM, gam_exit);
     signal(SIGPIPE, SIG_IGN);
 
     if (!gam_init_subscriptions()) {
