@@ -17,11 +17,11 @@
 
 #define MAX_REQUESTS 2048
 
-char pwd[250];
-char filename[MAXPATHLEN + 250];
-int interactive = 0;
+static char pwd[250];
+static char filename[MAXPATHLEN + 250];
+static int interactive = 0;
 
-struct testState {
+static struct testState {
     int connected;
     FAMConnection fc;
     int nb_requests;
@@ -313,7 +313,7 @@ processCommand(char *line, int no)
         testState.nb_requests++;
     } else if (!strcmp(command, "monfile")) {
         if (args != 2) {
-            fprintf(stderr, "mkdir line %d: lacks name\n", no);
+            fprintf(stderr, "monfile line %d: lacks name\n", no);
             return (-1);
         }
         if (arg[0] != '/')
@@ -353,6 +353,18 @@ processCommand(char *line, int no)
             return (-1);
         }
         printf("mkdir %s\n", arg);
+    } else if (!strcmp(command, "chmod")) {
+        if (args != 3) {
+            fprintf(stderr, "chmod line %d: lacks path and mode\n", no);
+            return (-1);
+        }
+        ret = chmod(arg, strtol (arg2, NULL, 8));
+        if (ret < 0) {
+            fprintf(stderr, "chmod line %d: failed to chmod %s to %s\n", no,
+                    arg, arg2);
+            return (-1);
+        }
+        printf("chmod %s to %s\n", arg, arg2);
     } else if (!strcmp(command, "mkfile")) {
         if (args != 2) {
             fprintf(stderr, "mkfile line %d: lacks name\n", no);
@@ -404,6 +416,18 @@ processCommand(char *line, int no)
             return (-1);
         }
         printf("rmfile %s\n", arg);
+	} else if (!strcmp(command, "move")) {
+		if (args != 3)
+		{
+			fprintf(stderr, "move line %d: lacks something\n", no);
+			return (-1);
+		}
+		ret = rename(arg, arg2);
+		if (ret < 0) {
+			fprintf(stderr, "move line %d: failed to move %s\n", no, arg);
+			return (-1);
+		}
+		printf("move %s %s\n", arg, arg2);
     } else if (!strcmp(command, "event")) {
         printEvent(no);
     } else if (!strcmp(command, "events")) {
